function [eigenanalyses]=Jnumrange_2x2(A,J,Tol,eigenanalyses)

% disp('Subrotine of the main file Jnumrange_main_routine.m');
% disp(' ')
% disp('Programmer: Ana Nata');
% disp('e-mail: anata@ipt.pt')
% disp(' ');
% disp('December 2013')
% disp(' ')
% disp('This subroutine make the plot of the J-numerical range')
% disp('of the 2x2 compression matrix, here denoted by A for simplification') 

%%
Z=eig(A); %eigenvalues of the matrix A 
eigenanalyses=eigenanalyses+1;
    if isequal(J,eye(size(J,2))) || isequal(J,-eye(size(J,2))) %Definite case
        B=transpose(conj(A)); 
        N=trace(B*A)-abs(Z(1,1))^2-abs(Z(2,1))^2; %length of the minor axis of the ellipse
        M=trace(B*A)-2*real(conj(Z(1,1))*Z(2,1)); %length of the major axis of the ellipse
            if abs(abs(M)-abs(N))<Tol %if M and N are almost equal  
                theta=0; 
            else
                tantheta=(imag(Z(2,1))-imag(Z(1,1)))/(real(Z(2,1))-real(Z(1,1)));
                theta=atan(tantheta); 
            end
        %Plot of the ellipse defined by the elliptical range theorem 
        c=(Z(1,1)+Z(2,1))/2; %center
        c1=real(c);c2=imag(c);
        a=sqrt(M)/2;b=sqrt(N)/2;
        t=linspace(0,2*pi,200);
        x=a*cos(t);
        y=b*sin(t);
        X=cos(theta)*x-sin(theta)*y+c1;
        Y=sin(theta)*x+cos(theta)*y+c2;
        plot(real(X),real(Y),'color','magenta')
        hold on
    elseif isequal(J,diag([1 -1])) || isequal(J,diag([-1 1])) %indefnite case
        alpha=(1/2)*(trace(A));
        B=J*transpose(conj(A))*J;
        M=abs(Z(1,1))^2+abs(Z(2,1))^2-trace(B*A); %transverse axis
        N=trace(B*A)-2*real(conj(Z(1,1))*Z(2,1)); %non-transverse axis
            if M>0 && N>0 && abs(M)>Tol && abs(N)>Tol
                tantheta=(imag(Z(2,1))-imag(Z(1,1)))/(real(Z(2,1))-real(Z(1,1)));
                theta=atan(tantheta);
                %Plot of the hyperbole defined by the elliptical range theorem  
                c=(Z(1,1)+Z(2,1))/2;
                c1=real(c);c2=imag(c);
                a=sqrt(N)/2;b=sqrt(M)/2;
                t=linspace(-4,4,200);
                x=a*cosh(t);
                y=b*sinh(t);
                xx=-a*cosh(t);
                X=cos(theta)*x-sin(theta)*y+c1;
                Y=sin(theta)*x+cos(theta)*y+c2;
                XX=cos(theta)*xx-sin(theta)*y+c1;
                YY=sin(theta)*xx+cos(theta)*y+c2;
                plot(real(X),real(Y),'color','cyan')
                hold on
                plot(real(XX),real(YY),'color','cyan')
                axis equal
                hold on
            elseif M>0 && abs(M)>Tol && abs(N)<=Tol
                if abs(A(1,2))==abs(A(2,1)) 
                    if imag(Z(2,1))==imag(Z(1,1)) %vertical line
                        a=[1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17];
                        b=[-20 -15 -10 -5 0 5 10 15 20 15 10 5 0 -5 -10 -15 -20]; 
                        plot(alpha*ones(length(a)),b,'cyan')
                        hold on
                        axis equal 
                    elseif real(Z(2,1))==real(Z(1,1)) %horizontal line
                        a=[-20 -15 -10 -5 0 5 10 15 20 15 10 5 0 -5 -10 -15 -20];
                        b=[1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17]; 
                        plot(a,alpha*ones(length(b)),'cyan')
                        hold on
                        axis equal
                    else   %line passing in alpha and perpendicular to the line segment defined by the eigenvalues
                        m1=(imag(Z(2,1))-imag(Z(1,1)))/(real(Z(2,1))-real(Z(1,1)));
                        m=-1/m1;
                        t=-10:0.5:10;
                        y=imag(alpha)+m*(t-real(alpha));
                        plot(t,y,'cyan')
                        hold on
                        axis equal
                    end
                else
                    text(0.18,0.55,'W_J(A) is the whole complex plane without a line)','Color','red');
                end
            elseif M>0 && N<0 && abs(M)>Tol && abs(N)>Tol
                text(0.18,0.55,'W_J(A) is the whole complex plane)','Color','red');
            elseif abs(M)<Tol && N>0 && abs(N)>Tol %line defined by the eigenvalues except the line segment with the end points are the eigenvalues
                if imag(Z(2,1))==imag(Z(1,1))  
                    if real(Z(1,1))<=real(Z(2,1)) 
                        vp1=Z(1,1);
                        vp2=Z(2,1);
                    else
                        vp1=Z(2,1);
                        vp2=Z(1,1);
                    end
                    a=[];
                        for t=1:17
                            a=[a real(vp1)-t+1];
                        end
                    b=[-20 -15 -10 -5 0 5 10 15 20 15 10 5 0 -5 -10 -15 -20]; 
                    plot(a,imag(vp1)*ones(length(b)),'cyan'); 
                    hold on
                    a=[];
                        for t=1:17
                            a=[a real(vp2)+t-1];
                        end
                    b=[-20 -15 -10 -5 0 5 10 15 20 15 10 5 0 -5 -10 -15 -20]; 
                    plot(a,imag(vp2)*ones(length(b)),'cyan'); 
                    hold on
                elseif real(Z(2,1))==real(Z(1,1)) 
                    if imag(Z(1,1))<=imag(Z(2,1))
                        vp1=Z(1,1);
                        vp2=Z(2,1);
                    else
                        vp1=Z(2,1);
                        vp2=Z(1,1);
                    end
                    b=[];
                        for t=1:17
                            b=[b imag(vp1)-t+1];
                        end
                    a=[-20 -15 -10 -5 0 5 10 15 20 15 10 5 0 -5 -10 -15 -20]; 
                    plot(real(vp1)*ones(length(a)),b,'cyan') 
                    hold on
                    axis equal 
                    b=[];
                        for t=1:17
                            b=[b imag(vp2)+t-1];
                        end
                    a = [-20 -15 -10 -5 0 5 10 15 20 15 10 5 0 -5 -10 -15 -20]; 
                    plot(real(vp2)*ones(length(a)),b,'cyan')
                    hold on
                    axis equal
                else    
                    if real(Z(1,1))<=real(Z(2,1))
                        vp1=Z(1,1);
                        vp2=Z(2,1);
                    else
                        vp1=Z(2,1);
                        vp2=Z(1,1);
                    end
                    t=-10:0;
                    x=t*real(vp1)+(1-t)*real(vp2);
                    y=t*imag(vp1)+(1-t)*imag(vp2);
                    plot(x,y,'cyan')
                    hold on
                    axis equal
                    t=1:10;
                    x=t*real(vp1)+(1-t)*real(vp2);
                    y=t*imag(vp1)+(1-t)*imag(vp2);
                    plot(x,y,'cyan')
                    hold on
                    axis equal
                end
            elseif abs(M)<Tol && abs(N)<Tol
                if abs(abs(A(1,1))-abs(A(2,2)))<Tol 
                    plot(real(alpha),imag(alpha),'.red','MarkerSize',10);
                else
                    text(0.18,0.55,'W_J(A) is a line without a point)','Color','red');
                end
            end
    end


