function Jnumrange_main_routine

clear all; close all; clc

disp('This is a program developed to plot the Krein space numerical range') 
disp(' ')
disp('Please feel free to use it for your study or research')
disp(' ')

disp('Programmer:')
disp('Ana Nata (Polytechnic Institute of Tomar, Portugal)')
disp('e-mail: anata@ipt.pt')
disp(' ')

disp('December 2013')
disp(' ');

disp('The algorithm is developed based on the Theorem 3.1 of the article')
disp('Computing the numerical range of Krein space operators') 
disp('submited by N. Bebiano, J. da Providncia, A. Nata and J.P. da Providncia')
disp('According to the Theorem 3.1 of this article, the Krein space numerical')
disp('range of any n by n (real or complex) matrix is the pseudo-convex hull')
disp('of a union of elliptical and hyperbolical discs.')
disp(' ')

disp('This program works also to the classical numerical range. For that, the user')
disp('need only to consider J=\pm I_n')
disp(' ')

disp(' This file is named as Jnumrange_main_routine.m');
disp('1. run matlab');
disp('2. input this file Jnumrange_main_routine.m and all the subroutines necessaries that');
disp('2. can be found on http://www.mat-uc.pt/~bebiano');
disp(' ')

disp('INPUT:'); 
disp('m = '); %enter a positive integer m>=3 that gives the number of elliptical
%and hyperbolical discs that are considered.
disp('N = '); %enter a positive integer N>=15 that gives the tolerance pi/(2^N) 
%considered in section 4.1 of the article.
disp('A = '); %enter a n-square (real or complex) matrix.')
disp('J = '); %enter a n-square involutive Hermitian matrix (i.e., J=J* and J^2=I_n)
%with signature (r,n-r), 0<=r<=n. By Sylvesters law of inertia, we can consider
%that J=I_r \oplus I_{n-r}.
disp(' ');

%% INPUTS example 
m=3;
N=15; 
A=[-1+1i 1 0 0;1 2+1i -1 0; 0 0 -1+1i 1;0 1 1 2+1i]; 
J=[1 0 0 0; 0 -1 0 0; 0 0 1 0; 0 0 0 -1];
%%
Tol=10^(-5);
m=round(m); %ensure that it is an integer
    if m<3  %ensure that it is greater or equal to 3
        m=3; 
    end 
N=round(N); %ensure that it is an integer   
    if N<15  %ensure that it is greater or equal to 15
        N=15; 
    end 
    
%% Checking the existence of some kind of errors
if ~isequal(size(J,1),size(J,2)) 
    error('J must be a square matrix');
end

if ~isequal(size(A,1),size(A,2))
    error('A must be a square matrix');
end

if ~isequal(size(J),size(A)) && ~isequal(size(J),size(A))
    error('A and J must have the same size');
end

for r=1:size(J,2)
    for s=1:size(J,2)
        if r~=s && J(r,s)~=0 || J(r,r)~=1 && J(r,r)~=-1
            error('J must be an Hermitian involutive matrix');
        end
    end
end
%% start the run time
tic 
%% Test if the user choose the classical or the Krein space numerical range
if isequal(J,eye(size(J,2))) || isequal(J,-eye(size(J,2)))
    definite=1; %definite (or classical) numerical range 
    if m<=15
        m=15;
    end
    disp('In the definite case, m must be at least 8. So we consider m=15')
    disp(' ');
else
    definite=0; %indefinite (or Krein space) numerical range 
end
%% Test if A is J-escalar
B=J*A;
    if (isequal(B,B(1,1)*eye(size(J,2)))==1) || isequal(A, A(1,1)*eye(size(J,2)))==1
        plot(real(B(1,1)),imag(B(1,1)),'.blue', 'MarkerSize',5)    
        xlabel('Real Axis');
        ylabel('Imaginary Axis');
        axis equal
        disp('The matriz is J-scalar, so the J-numerical range is a singleton')
        toc %end the run time
        return;
    end
%% Test if W_J(A) is non-degenerate
realA=0;
    for w=1:size(A,1)
        for e=1:size(A,2)
            if imag(A(w,e))~=0
                realA=1; 
                break
            end
        end
        if realA==1
            break
        end
    end
eigenanalyses=0;
    if definite==1
        if realA~=0
            [aux1,theta1,thetaM,center,eigenanalyses,k1,k2,kc,s1,s2]=center_complex(A,J,Tol,eigenanalyses);
        else
            aux1=0;
        end
    else
        [v1 v2 interla0 aux0]=theta1_e_thetaM(A,J,Tol,0); %test if 0 is an admissible direction    
        eigenanalyses=eigenanalyses+1;
            if realA==0 && interla0==2 && aux0==1 %non-degenerate case, when A is real
                aux1=0;
            elseif realA==0 && (interla0~=2 || aux0~=1) %degenerate case, when A is real
                aux1=1;
                text(0.25,0.6,'W_J(A) is a degenerate case, i.e., W_J(A) is','Color','red');
                text(0.25,0.55,'a line (maybe without a point) or the whole','Color','red');
                text(0.25,0.5,'complex plane (maybe without a line).','Color','red');
            else
                [aux1,theta1,thetaM,center,eigenanalyses,k1,k2,kc,s1,s2]=center_complex(A,J,Tol,eigenanalyses);
                    if aux1==1
                        text(0.25,0.6,'W_J(A) is a degenerate case, i.e., W_J(A) is','Color','red');
                        text(0.25,0.55,'a line (maybe without a point) or the whole','Color','red');
                        text(0.25,0.5,'complex plane (maybe without a line).','Color','red');
                    end
            end
    end
%% Computation of the numerical range    
if aux1==0 || definite==1
    % Computation of theta_{min} and theta_{max} (here denoted by theta1 and thetaM, respectivelly) 
    if realA==0 %A is a real matrix
        if definite==1
            thetaM=0;
            theta1=pi;
        else
            thetaM=pi/2;
            [theta1,thetaM,eigenanalyses]=refinar_real(A,J,Tol,thetaM,N,eigenanalyses);
        end
    else %A is a complex matrix
        if definite==1 
            thetaM=center;
            theta1=center+pi;
        else        
            %Refine thetaM like in subsection 4.1  
            [thetaM,eigenanalyses]=refinar_thetaM_complex(A,J,Tol,thetaM,k1,s1,center,N,eigenanalyses);
            %Refine theta1 like in subsection 4.1 
            [theta1,eigenanalyses]=refinar_theta1_complex(A,J,Tol,theta1,k2,s2,center,N,eigenanalyses);
        end
    end
    [eigenanalyses,DD1,DD2,X1,Y1]=regulargrid(A,J,m,Tol,theta1,thetaM,eigenanalyses,definite);
    eigenanalyses_mat2=0;
        if (isempty(DD1)==0 && isempty(DD2)==0 && definite==0) || definite==1
            for r=2:size(X1,2)
                uu1=X1(:,r);
                uu2=X1(:,r-1);
                [eigenanalyses_mat2]=compression_Jnumrange(uu1,uu2,A,J,Tol,definite,eigenanalyses_mat2);
                vv1=Y1(:,r);
                vv2=Y1(:,r-1);
                [eigenanalyses_mat2]=compression_Jnumrange(vv1,vv2,A,J,Tol,definite,eigenanalyses_mat2);
            end    
        end 
    uu1=X1(:,1);
    uu2=Y1(:,1);
    [eigenanalyses_mat2]=compression_Jnumrange(uu1,uu2,A,J,Tol,definite,eigenanalyses_mat2);
    vv1=Y1(:,size(Y1,2));
    vv2=X1(:,size(X1,2));
    [eigenanalyses_mat2]=compression_Jnumrange(vv1,vv2,A,J,Tol,definite,eigenanalyses_mat2);
    xlabel('Real Axis');
    ylabel('Imaginary Axis');
    axis equal
    title('')
    toc %end the run time
    disp(' ');
    fprintf('eigenanalyses = %d \n', eigenanalyses)
    disp(' ');
    fprintf('theta_min= %s \n',theta1)
    disp(' ');
    fprintf('theta_max= %s \n',thetaM)    
end 

